import { EventHandler, guid, Message } from './common';
import type { IEvent } from './common';
export { IEvent, EventHandler, guid, Message };
import * as MQTT from './paho-mqtt';
export declare type MQTTError = MQTT.MQTTError;
export declare type Qos = MQTT.Qos;
export declare type TraceFunction = MQTT.TraceFunction;
/**
 * Attributes used with a connection.
 */
export interface ConnectionOptions {
    /**
     * If the connect has not succeeded within this number of seconds, it is deemed to have failed.
     * @default The default is 30 seconds.
     */
    timeout?: number;
    /** Authentication username for this connection. */
    userName?: string;
    /** Authentication password for this connection. */
    password?: string;
    /** Sent by the server when the client disconnects abnormally. */
    willMessage?: Message;
    /**
     * The server disconnects this client if there is no activity for this number of seconds.
     * @default The default value of 60 seconds is assumed if not set.
     */
    keepAliveInterval?: number;
    /**
     * If true(default) the client and server persistent state is deleted on successful connect.
     * @default true
     */
    cleanSession?: boolean;
    /** If present and true, use an SSL Websocket connection. */
    useSSL?: boolean;
    /** Passed to the onSuccess callback or onFailure callback. */
    invocationContext?: any;
    /**
     * Specifies the mqtt version to use when connecting
     * <dl>
     *     <dt>3 - MQTT 3.1</dt>
     *     <dt>4 - MQTT 3.1.1 (default)</dt>
     * </dl>
     * @default 4
     */
    mqttVersion?: 3 | 4;
    /**
     * If set to true, will force the connection to use the selected MQTT Version or will fail to connect.
     */
    mqttVersionExplicit?: boolean;
    /**
     * Sets whether the client will automatically attempt to reconnect to the server if the connection is lost.
     * <dl>
     *     <dt>If set to false, the client will not attempt to automatically reconnect to the server in
     *         the event that the connection is lost.</dt>
     *     <dt>If set to true, in the event that the connection is lost, the client will attempt to
     *         reconnect to the server. It will initially wait 1 second before it attempts to reconnect,
     *         for every failed reconnect attempt, the delay will double until it is at 2 minutes at which
     *         point the delay will stay at 2 minutes.</dt>
     * </dl>
     * @default false
     */
    reconnect?: boolean;
    /**
     * If present this contains either a set of hostnames or fully qualified
     * WebSocket URIs (ws://example.com:1883/mqtt), that are tried in order in place of the host and port
     * paramater on the construtor. The hosts are tried one at at time in order until one of then succeeds.
     */
    hosts?: string[];
    /**
     * If present the set of ports matching the hosts. If hosts contains URIs, this property is not used.
     */
    ports?: number[];
}
export interface BaseClientOptions {
    /**
     * the Messaging client identifier, between 1 and 23 characters in length.
     * If not set, a random UID is used.
     */
    clientId?: string;
}
export interface HostUriClientOptions extends BaseClientOptions {
    /** the address of the messaging server as a fully qualified WebSocket URI */
    hostUri: string;
}
export interface HostClientOptions extends BaseClientOptions {
    /** the address of the messaging server as a DNS name or dotted decimal IP address. */
    host: string;
    /** the port number to connect to */
    port: number;
    /**
     * the path on the host to connect to - only used if host is not a URI.
     * @default '/mqtt'
     */
    path?: string;
}
export declare type ClientOptions = HostClientOptions | HostUriClientOptions;
export interface OnConnectedParams {
    reconnect: boolean;
    uri: string;
}
export interface SubscribeOptions {
    qos?: Qos;
    timeout?: number;
}
export interface OnSubscribedParams {
    grantedQos: Qos;
}
export interface UnsubscribeOptions {
    timeout?: number;
}
export declare enum ConnectionState {
    CONNECTED = 0,
    CONNECTING = 1,
    DISCONNECTED = 2
}
export declare class MQTTClient {
    private mqttClient;
    private host;
    private hostUri;
    private port;
    private path;
    clientId: string;
    get connected(): boolean;
    private _connectionState;
    get connectionState(): ConnectionState;
    private connectionSuccess;
    private mqttConnected;
    private connectionFailure;
    private connectionLost;
    private subscribeSuccess;
    private subscribeFailure;
    private unsubscribeSuccess;
    private unsubscribeFailure;
    private messageArrived;
    private messageDelivered;
    constructor(options: ClientOptions);
    get onConnectionSuccess(): IEvent<void>;
    get onConnected(): IEvent<OnConnectedParams>;
    get onConnectionFailure(): IEvent<MQTTError>;
    get onConnectionLost(): IEvent<MQTTError>;
    get onSubscribeSuccess(): IEvent<OnSubscribedParams>;
    get onSubscribeFailure(): IEvent<MQTTError>;
    get onUnsubscribeSuccess(): IEvent<void>;
    get onUnsubscribeFailure(): IEvent<MQTTError>;
    get onMessageArrived(): IEvent<Message>;
    get onMessageDelivered(): IEvent<Message>;
    connect(connectOptions?: ConnectionOptions): Promise<void>;
    disconnect(): void;
    subscribe(topic: string, subscribeOpts?: SubscribeOptions): Promise<OnSubscribedParams>;
    unsubscribe(topic: string, opts?: UnsubscribeOptions): Promise<void>;
    publish(message: Message): void;
    setTraceFunction(f: TraceFunction): void;
    startTrace(): void;
    stopTrace(): void;
    private defer;
}
